@@ -5,9 +5,12 @@ Tasks used to compile treasury measures
 import datetime
 import transaction
 
+from dateutil.relativedelta import relativedelta
+
 from pyramid_celery import celery_app
 from sqlalchemy import or_, desc
 from collections import OrderedDict
+from datetime import date
 
 from endi_base.models.base import DBSESSION
 from endi_base.mail import send_mail
@@ -23,6 +26,7 @@ from endi.models.accounting.treasury_measures import (
 )
 from endi_celery.conf import (
     get_recipients_addresses,
+    get_setting,
 )
 from endi.models.accounting.income_statement_measures import (
     IncomeStatementMeasure,
@@ -30,6 +34,10 @@ from endi.models.accounting.income_statement_measures import (
     IncomeStatementMeasureType,
     IncomeStatementMeasureTypeCategory,
 )
+from endi.models.accounting.accounting_closures import (
+    AccountingClosure,
+)
+from endi.models.config import get_config
 
 from endi_celery.tasks import utils
 
@@ -389,7 +397,8 @@ class BaseMeasureCompiler(object):
                     grid_item.store_measure(measure)
                 else:
                     logger.debug("Updating the order of the given measure {} :\
- was {} becomes {}".format(measure.label, measure.order, measure_type.order))
+                            was {} becomes {}".format(measure.label, measure.order,
+                            measure_type.order))
                     measure.order = measure_type.order
                     self.session.merge(measure)
 
@@ -660,6 +669,136 @@ def send_error(request, mail_addresses, type_label, message):
         except:
             logger.exception("Error sending email")
 
+def _collect_operations(upload_id, data_type, upload_date):
+    """
+    Return the operations needed.
+    Collect the operations ("écritures") according to the "exercice fiscal".
+    If the "exercice" has been set "closed" in enDI we don't collect the
+    operation of former year (because à-nouveau operation have been done by
+    the accountant.
+    For now, this function is only for the treasury meausures, in the future
+    it might also be for the income statement measure.
+    """
+    logger.debug(u"Collecting operations for {} upload".format(upload_id))
+
+    # Reference year for collecting data
+    current_year = upload_date.year
+    former_closure_year = current_year - 1
+
+    # Get config from main app
+    config = get_config()
+
+    closure_day = 0
+    closure_month = 0
+    if(config.get("accounting_closure_day") != None and
+            config.get("accounting_closure_month") != None):
+        closure_day = int("0" + str(config.get("accounting_closure_day")))
+        closure_month = int("0" + str(config.get("accounting_closure_month")))
+
+    # If days and month not set, assume that closure is at the end of the year
+    if(closure_day == 0 or closure_month == 0):
+        logger.info(u"Accounting closure day or month not set, assuming closure\
+                is on 31/12")
+        closure_day = 31
+        closure_month = 12
+
+    # Find exercice begining day and month
+    temp_date = date(current_year, closure_month, closure_day) +\
+        datetime.timedelta(days=1)
+    begining_day = temp_date.day
+    begining_month = temp_date.month
+
+    ## Collecting operation for current exercice fiscal
+    current_exercice_begining_date = date(current_year, begining_month,
+            begining_day)
+    current_exercice_end_date = date(current_year, closure_month,
+            closure_day)
+
+    # If closure day is not 31/12 we have an exercice overlaping two years
+    if not (closure_day == 31 and closure_month == 12):
+        # Is today after the closure this year ?
+        this_month = datetime.datetime.now().month
+        this_day = datetime.datetime.now().day
+        today_is_after_the_closure = None
+
+        if this_month > closure_month:
+            today_is_after_the_closure = True
+        elif this_month == closure_month:
+            if(this_day > closure_day):
+                today_is_after_the_closure = True
+            else :
+                today_is_after_the_closure = False
+        else:
+           today_is_after_the_closure = False
+
+        logger.info("Today is after the closure this year :\
+                {}".format(today_is_after_the_closure))
+
+        # If so or not we set the corresponding exercice dates
+        if today_is_after_the_closure:
+            current_exercice_begining_date =\
+                    current_exercice_begining_date.replace(year=current_year)
+            current_exercice_end_date = current_exercice_end_date +\
+                    relativedelta(years=1)
+        else:
+            current_exercice_begining_date = current_exercice_begining_date -\
+                    relativedelta(years=1)
+            current_exercice_end_date =\
+                    current_exercice_end_date.replace(year=current_year)
+
+    logger.info("Current exercice start the {} and ends the\
+            {}".format(current_exercice_begining_date,
+                current_exercice_end_date))
+
+    current_exercice_operations = AccountingOperation.query().filter(
+        AccountingOperation.date.between(current_exercice_begining_date,
+            current_exercice_end_date)
+    ).all()
+
+    logger.debug(u"{} operations\
+            collected".format(len(current_exercice_operations)))
+
+    ## Adding last exercice operations if needed (dans les cas ou les report
+    ## d'à nouveau n'ont pas encore été fait)
+    # Only for synchronised  operations (remontée automatique)
+    if(data_type != AccountingOperationUpload.SYNCHRONIZED_ACCOUNTING):
+        return current_exercice_operations
+
+    # Look into AccountingClosure model to check if closure for current year
+    # has been done
+    former_year_closure = AccountingClosure.query().filter_by(
+        year=former_closure_year
+    ).first()
+
+    closure_is_done = False
+    if former_year_closure:
+        closure_is_done = former_year_closure.active
+
+    logger.info(u"{} closure is done : {}".format(former_closure_year, closure_is_done))
+
+    if not closure_is_done:
+        ## Find date of last exercice
+        last_exercice_begining_date = current_exercice_begining_date -\
+            relativedelta(years=1)
+        last_exercice_end_date = current_exercice_end_date -\
+            relativedelta(years=1)
+
+        logger.info("Last exercice start the {} and ends the\
+                {}".format(last_exercice_begining_date,
+                    last_exercice_end_date))
+
+        # Select operations from last "exercice"
+        last_exercice_operations = AccountingOperation.query().filter(
+            AccountingOperation.date.between(last_exercice_begining_date,
+                last_exercice_end_date)
+        ).all()
+
+        logger.info(u"Appending {} operations from last\
+                exercice".format(len(last_exercice_operations)))
+
+        return last_exercice_operations + current_exercice_operations
+
+    return current_exercice_operations
 
 @celery_app.task(bind=True)
 def compile_measures_task(self, upload_id, grid_type=None):
@@ -681,9 +820,21 @@ def compile_measures_task(self, upload_id, grid_type=None):
                 upload_id
             )
         )
-    operations = AccountingOperation.query().filter_by(
-        upload_id=upload_id
-    ).all()
+
+    operations = None
+    if grid_type == "treasury":
+        # We use created_at date, because his year is necessarily
+        # into the requested exercice for computing the measures
+        operations = _collect_operations(upload_id, upload.filetype,
+            upload.created_at.date())
+    else:
+        # Pour l'instant _collect_operations ne concerne que les états de
+        # trésorerie, à terme quand nous voudrons présenter les comptes de
+        # résultat par exercice fiscal elle pourra être également utilisé
+        # pour les comptes de résultat.
+        operations = AccountingOperation.query().filter_by(
+            upload_id=upload_id
+        ).all()
 
     mail_addresses = get_recipients_addresses(self.request)
     compiler_factories = get_measure_compilers(upload.filetype, grid_type)
